# Beaker
#
# Copyright (C) 2010 dcallagh@redhat.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import unittest
import logging
import re
from turbogears.database import session

from bkr.server.model import Job, TaskResult, RecipeTaskResult
from bkr.inttest.server.selenium import SeleniumTestCase, WebDriverTestCase
from bkr.inttest.server.webdriver_utils import login, is_text_present
from bkr.inttest import data_setup, get_server_base
from bkr.inttest.assertions import assert_sorted

class TestRecipesDataGrid(SeleniumTestCase):

    log = logging.getLogger(__name__ + '.TestRecipesIndex')

    # tests in this class can safely share the same firefox session
    @classmethod
    def setUpClass(cls):
        # create a bunch of jobs
        cls.user = user = data_setup.create_user(password='password')
        arches = [u'i386', u'x86_64', u'ia64']
        distro_names = [u'DAN5-Server-U5', u'DAN5-Client-U5',
                u'DAN6-Server-U1', u'DAN6-Server-RC3']
        for arch in arches:
            for distro_name in distro_names:
                distro = data_setup.create_distro(name=distro_name, arch=arch)
                data_setup.create_job(owner=user, distro=distro)
                data_setup.create_completed_job(owner=user, distro=distro)
        session.flush()

        # XXX we could save a *lot* of time by reusing Firefox instances across tests
        cls.selenium = sel = cls.get_selenium()
        sel.start()

        # log in
        sel.open('')
        sel.click('link=Login')
        sel.wait_for_page_to_load('30000')
        sel.type('user_name', user.user_name)
        sel.type('password', 'password')
        sel.click('login')
        sel.wait_for_page_to_load('30000')

    @classmethod
    def tearDownClass(cls):
        cls.selenium.stop()

    # see https://bugzilla.redhat.com/show_bug.cgi?id=629147
    def check_column_sort(self, column):
        sel = self.selenium
        sel.open('recipes/mine')
        sel.click('//table[@id="widget"]/thead/th[%d]//a[@href]' % column)
        sel.wait_for_page_to_load('30000')
        row_count = int(sel.get_xpath_count(
                '//table[@id="widget"]/tbody/tr/td[%d]' % column))
        self.assertEquals(row_count, 24)
        cell_values = [sel.get_table('widget.%d.%d' % (row, column - 1)) # zero-indexed
                       for row in range(0, row_count)]
        assert_sorted(cell_values)

    def test_can_sort_by_whiteboard(self):
        self.check_column_sort(2)

    def test_can_sort_by_arch(self):
        self.check_column_sort(3)

    def test_can_sort_by_system(self):
        self.check_column_sort(4)

    def test_can_sort_by_distro(self):
        self.check_column_sort(5)

    def test_can_sort_by_status(self):
        self.check_column_sort(7)

    def test_can_sort_by_result(self):
        self.check_column_sort(8)

    # this version is different since the cell values will be like ['R:1', 'R:10', ...]
    def test_can_sort_by_id(self):
        column = 1
        sel = self.selenium
        sel.open('recipes/mine')
        sel.click('//table[@id="widget"]/thead/th[%d]//a[@href]' % column)
        sel.wait_for_page_to_load('30000')
        row_count = int(sel.get_xpath_count(
                '//table[@id="widget"]/tbody/tr/td[%d]' % column))
        self.assertEquals(row_count, 24)
        cell_values = []
        for row in range(0, row_count):
            raw_value = sel.get_table('widget.%d.%d' % (row, column - 1)) # zero-indexed
            m = re.match(r'R:(\d+)$', raw_value)
            assert m.group(1)
            cell_values.append(int(m.group(1)))
        assert_sorted(cell_values)

class TestRecipeView(WebDriverTestCase):

    def setUp(self):
        self.user = user = data_setup.create_user(display_name=u'Bob Brown',
                password='password')
        self.system_owner = data_setup.create_user()
        self.system = data_setup.create_system(owner=self.system_owner, arch=u'x86_64')
        distro = data_setup.create_distro(arch=u'x86_64')
        self.job = data_setup.create_completed_job(owner=user, distro=distro, server_log=True)
        for recipe in self.job.all_recipes:
            recipe.system = self.system
        session.flush()
        self.browser = self.get_browser()
        login(self.browser, user=user.user_name, password='password')

    def tearDown(self):
        self.browser.quit()

    def go_to_recipe_view(self, recipe):
        b = self.browser
        b.get(get_server_base() + 'recipes/mine')
        b.find_element_by_link_text(recipe.t_id).click()

    # https://bugzilla.redhat.com/show_bug.cgi?id=623603
    # see also TestSystemView.test_can_report_problem
    def test_can_report_problem(self):
        b = self.browser
        recipe = list(self.job.all_recipes)[0]
        self.go_to_recipe_view(recipe)
        b.find_element_by_link_text('Report problem with system').click()
        self.assertEqual(b.title,
                'Report a problem with %s' % self.system.fqdn)

    def test_log_url_looks_right(self):
        b = self.browser
        some_job = self.job
        r = some_job.recipesets[0].recipes[0]
        self.go_to_recipe_view(r)
        b.find_element_by_id("all_recipe_%s" % r.id).click()
        rt_log_server_link = b.find_element_by_xpath("//tr[@class='even pass_recipe_%s recipe_%s']//td[position()=4]//a" % (r.id, r.id)).get_attribute('href')
        rt_log = r.tasks[0].logs[0]
        self.assert_(rt_log_server_link == rt_log.server + '/' + rt_log.filename)
        b.find_element_by_id("logs_button_%s" % r.id).click()
        r_server_link = b.find_element_by_xpath("//table[@class='show']/tbody//tr[position()=6]/td/a").get_attribute('href')
        self.assert_(r_server_link == r.logs[0].server + '/' + r.logs[0].filename)

    def test_task_pagination(self):
        num_of_tasks = 35
        the_tasks = [data_setup.create_task() for t in range(num_of_tasks)]
        the_recipe = data_setup.create_recipe(task_list=the_tasks)
        the_job = data_setup.create_job_for_recipes([the_recipe], owner=self.user)
        session.flush()

        b = self.browser
        self.go_to_recipe_view(the_recipe)
        b.find_element_by_id("all_recipe_%s" % the_recipe.id).click()
        for t in the_job.recipesets[0].recipes[0].tasks:
            self.assertTrue(is_text_present(b, "T:%s" %t.id))

    # https://bugzilla.redhat.com/show_bug.cgi?id=751330
    def test_fetching_large_results_is_not_too_slow(self):
        tasks = [data_setup.create_task() for _ in range(700)]
        recipe = data_setup.create_recipe(task_list=tasks)
        pass_ = TaskResult.by_name(u'Pass')
        for rt in recipe.tasks:
            rt.results = [RecipeTaskResult(path=u'result_%d' % i,
                    result=pass_, score=i) for i in range(10)]
        job = data_setup.create_job_for_recipes([recipe], owner=self.user)
        session.flush()

        b = self.browser
        self.go_to_recipe_view(recipe)
        b.find_element_by_id('all_recipe_%s' % recipe.id).click()
        # Let's set a wait time of 30 seconds and try to find the results table.
        # If the server is taking too long to return our results,
        # we will get a NoSuchElementException below.
        b.implicitly_wait(30)
        b.find_element_by_xpath('//div[@id="task_items_%s"]//table' % recipe.id)
